/*
 *  dialogInt.h -- private declarations for the dialog library
 *
 *  Savio Lam (lam836@cs.cuhk.hk)
 *  Alessandro Rubini (rubini@ipvvis.unipv.it)
 *     a few others contributed
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#ifdef ultrix
#  include <cursesX.h>
#else
#  ifdef HAVE_NCURSES_H
#    include <ncurses.h>
#  else
#    include <curses.h>
#  endif
#endif

/*
 * Change these if you want
 */
#define DIALOG_USE_SHADOW FALSE
#define DIALOG_USE_COLORS TRUE


#define DIALOG_ESC 27
#define DIALOG_TAB 9
#define DIALOG_MAX_LEN 2048
#define DIALOG_BUF_SIZE (10*1024)

#ifndef min
#  define min(x,y) ((x)<(y)?(x):(y))
#endif
#ifndef max
#  define max(x,y) ((x)>(y)?(x):(y))
#endif

/*
 * Support for PDEBUG((stderr,"format %i %s\n",i,s))
 *   "PDEBUGG(())" will always expand to nothing,
 *   "PDEBUG(())" will only expand if DEBUG_DIALOG is defined
 * Note that I print debug messages to stderr
 */

#ifndef PDEBUG
#  ifdef DEBUG_DIALOG
#    define PDEBUG(data) fprintf data
#  else
#    define PDEBUG(data)
#  endif
#endif

#ifndef PDEBUGG
#  define PDEBUGG(data)
#endif

#ifndef HAVE_NCURSES
#ifndef ACS_ULCORNER
#define ACS_ULCORNER '+'
#endif
#ifndef ACS_LLCORNER
#define ACS_LLCORNER '+'
#endif
#ifndef ACS_URCORNER
#define ACS_URCORNER '+'
#endif
#ifndef ACS_LRCORNER
#define ACS_LRCORNER '+'
#endif
#ifndef ACS_HLINE
#define ACS_HLINE '-'
#endif
#ifndef ACS_VLINE
#define ACS_VLINE '|'
#endif
#ifndef ACS_LTEE
#define ACS_LTEE '+'
#endif
#ifndef ACS_RTEE
#define ACS_RTEE '+'
#endif
#ifndef ACS_UARROW
#define ACS_UARROW '^'
#endif
#ifndef ACS_DARROW
#define ACS_DARROW 'v'
#endif
#endif

#if 0  /* XXX These make nicer symbols, but are a bad hack. */
#undef  ACS_UARROW
#define ACS_UARROW '^'
#undef  ACS_DARROW
#define ACS_DARROW 'v'
#endif
/* 
 * Attribute names
 */
#define screen_attr                   attributes[0]
#define shadow_attr                   attributes[1]
#define dialog_attr                   attributes[2]
#define title_attr                    attributes[3]
#define border_attr                   attributes[4]
#define button_active_attr            attributes[5]
#define button_inactive_attr          attributes[6]
#define button_key_active_attr        attributes[7]
#define button_key_inactive_attr      attributes[8]
#define button_label_active_attr      attributes[9]
#define button_label_inactive_attr    attributes[10]
#define inputbox_attr                 attributes[11]
#define inputbox_border_attr          attributes[12]
#define searchbox_attr                attributes[13]
#define searchbox_title_attr          attributes[14]
#define searchbox_border_attr         attributes[15]
#define position_indicator_attr       attributes[16]
#define menubox_attr                  attributes[17]
#define menubox_border_attr           attributes[18]
#define item_attr                     attributes[19]
#define item_selected_attr            attributes[20]
#define tag_attr                      attributes[21]
#define tag_selected_attr             attributes[22]
#define tag_key_attr                  attributes[23]
#define tag_key_selected_attr         attributes[24]
#define check_attr                    attributes[25]
#define check_selected_attr           attributes[26]
#define uarrow_attr                   attributes[27]
#define darrow_attr                   attributes[28]

/* number of attributes */
#define ATTRIBUTE_COUNT               29

/*
 * Global variables
 */
#ifdef HAVE_NCURSES
extern bool use_colors;
extern bool use_shadow;
#endif

extern chtype attributes[];

/*
 * Silly macros for the arrows
 */

#define DIALOG_DOWNARROW(win,y,x) do { \
                  wattrset((win),darrow_attr); \
                  wmove((win),(y),(x));        \
		  waddch((win),ACS_DARROW); waddstr((win),"(+)"); } while (0)

#define DIALOG_UPARROW(win,y,x) do { \
                  wattrset((win),uarrow_attr); \
                  wmove((win),(y),(x));        \
                  waddch((win),ACS_UARROW); waddstr((win),"(-)"); } while (0)

#define DIALOG_NOARROW(win,y,x) do { \
	          int i; \
		  wattrset((win),menubox_attr); \
		  wmove((win),(y),(x));         \
	          for (i=0; i<4; i++)           \
		    waddch((win),ACS_HLINE); } while(0) 


/*
 * Prototypes for internal functions
 */

#ifdef HAVE_NCURSES
void    _dialog_create_rc (const char *filename);
int     _dialog_parse_rc (void);
void    _dialog_draw_shadow (WINDOW *win, int y, int x, int height, int width);
#endif

int     _dialog_setup (struct Dialog_desc *desc);
void    _dialog_print_autowrap (WINDOW * win, const char *prompt,
		int width, int y, int x);
int     _dialog_print_buttons (struct Dialog_desc *desc);
void    _dialog_draw_box (WINDOW * win, int y, int x, int height, int width,
		chtype box, chtype border);
void    _dialog_attr_clear (WINDOW * win, int height, int width, chtype attr);


/*
 * The following stuff is needed for mouse support
 */

#ifdef CFG_NOMOUSE

#define _dialog_m_open()                                     /* nothing */
#define _dialog_m_close()                                    /* nothing */
#define _dialog_mkregion(y,x,height,width,code)              /* nothing */
#define _dialog_mkbigregion(y,x,height,width,nitems,th,mode) /* nothing */
#define _dialog_setbase(x,y)                                 /* nothing */
#define _dialog_m_cleanup()                                  /* nothing */
#define _dialog_wgetch(w)                                    wgetch(w)

#else

void _dialog_m_open (void);
void _dialog_m_close (void);
void _dialog_mkregion (int y, int x, int height, int width, int code);
void _dialog_mkbigregion (int y, int x, int height, int width, int nitems,
			int th, int mode);
void _dialog_setbase (int x, int y);
void _dialog_m_cleanup (void);

int Gpm_Wgetch (WINDOW * w);
#define _dialog_wgetch(w) Gpm_Wgetch(w)

#endif

#define _dialog_mkbutton(y,x,len,code) _dialog_mkregion(y,x,1,len,code)

/*
 * This is the base for fictious keys, which activate
 * the buttons.
 *
 * Mouse-generated keys are the following:
 *   -- the first 32 are used as numbers, in addition to '0'-'9'
 *   -- the lowercase are used to signal mouse-enter events (M_EVENT + 'o')
 *   -- uppercase chars are used to invoke the button (M_EVENT + 'O')
 */
#define DIALOG_M_EVENT (KEY_MAX+1)



/**** Local Variables:  ****/
/**** tab-width:8       ****/
/**** c-indent-level:4  ****/
/**** c-label-offset:-2 ****/
/**** End:              ****/
