/*
 *  list.c -- implements the checklist, radiolist and menu boxes
 *
 *  Originally by Savio Lam (lam836@cs.cuhk.hk)
 *            and Stuart Herbert (S.Herbert@sheffield.ac.uk)
 *  Completely reworked by Alessandro Rubini (rubini@ipvvis.unipv.it).
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "dialog.h"
#include "dialogInt.h"

static int list_width, list_height, check_x, item_x;
static Dialog_list *curr, *list, *first, *last, *new, *prevradio;


/*
 * Print list item
 */
static void
print_item (WINDOW * win, int type, Dialog_list *item)
{
    int i, y=item->index-first->index;

    /* clear previous item */
    wattrset (win, menubox_attr);
    wmove (win, y, 0);
    for (i = 0; i < list_width; i++)
      waddch (win, ' ');

    wmove (win, y, check_x);
    if (type != DIALOG_MENU) {
	wattrset (win, item==curr ? check_selected_attr : check_attr);
	if (type == DIALOG_CHECK)
	  wprintw (win, "[%c]", item->selected ? 'X' : ' ');
	else
	  wprintw (win, "(%c)", item->selected ? 'X' : ' ');
	wattrset (win, menubox_attr);
	waddch (win, ' ');
    }
    /*
     * Missing: the hotkey is not the first
     */
    wattrset (win, item==curr ? tag_key_selected_attr : tag_key_attr);
    waddch (win, item->tag[0]);
    wattrset (win, item==curr ? tag_selected_attr : tag_attr);
    waddstr (win, item->tag + 1);
    wmove (win, y, item_x);
    wattrset (win, item==curr ? item_selected_attr : item_attr);
    if (item->text)
      waddstr (win, item->text);
}

/*
 * Display a dialog box with a list of options that can be turned on or off.
 * This function accounts for checklist, radiolist, menubox
 */
int
dialog_list (struct Dialog_desc *desc)
{
    int i, x, y, cur_x, cur_y, box_x, box_y, key;
    int curs_x, curs_y;
    int button = desc->selected;
    WINDOW *dialog, *subw;
    Dialog_butt *bptr;

    int item_no, x_offset, max_choice;

    list=desc->clientdata;
    list_height=desc->xtra;
    switch(desc->type) {
      case DIALOG_RADIO:
      case DIALOG_CHECK:
	x_offset=6; break;
      case DIALOG_MENU:
	x_offset=0; break;
      default:
	return DIALOG_ERROR;
    }
	  
    desc->hlinenum=1;
    desc->hline[0]=-3;
    if (!desc->buttons) {
	dialog_defaultbuttons(desc,"  Ok  :Cancel");
    }
    
    _dialog_setup(desc);
    dialog = desc->win;

    /*
     * Hmm.... the prompt has already been written, but the cursor
     * has already moved away, while drawing buttons. So I need to
     * save the cursor position, redraw the prompt, and restore the cursor.
     * This must be fixed when automagic positioning is implemented.
     */

    getyx (dialog, curs_y, curs_x);

    if (desc->prompt)
      _dialog_print_autowrap (dialog, desc->prompt, desc->wid - 2, 1, 3);

    /* horrible */
    x = (COLS - desc->wid) / 2;
    y = (LINES - desc->hei) / 2;

    getyx (dialog, cur_y, cur_x);
    box_y = cur_y + 1;
    box_x = 2;
    list_width = desc->wid - 6;

    /* create new window for the list */
    if (list_height==0) list_height=desc->hei-cur_y-6;
    subw = subwin (dialog, list_height, list_width,
		   y + box_y + 1, x + box_x + 1);
    if (!subw) {
	fprintf(stderr,"failed to create subwindow\n");
	return DIALOG_ERROR;
    }
    keypad (subw, TRUE);

    /* draw a box around the subw items */
    _dialog_draw_box (dialog, box_y, box_x, list_height + 2, list_width + 2,
	      menubox_border_attr, menubox_attr);

    /* Scan the list to find max length, and item_no */
    check_x=item_x=item_no=0;
    for (curr=list; curr; curr=curr->next) {
	item_no++;
	check_x=max(check_x,strlen(curr->tag));
	if (curr->text)
	  item_x=max(item_x,strlen(curr->text));
	prevradio=curr;
    }
    max_choice = min (list_height, item_no);

    if (desc->type==DIALOG_RADIO) { /* prevradio is now the last, scan backw */
	for (curr=prevradio; curr; curr=curr->prev) {
	    if (curr->selected) {
		prevradio->selected=DIALOG_B_DISABLED;
		curr->selected=DIALOG_B_ENABLED; /* it may be the same */
		prevradio=curr;
	    }
	}
	if (!prevradio->selected) /* nothing preselected, use the first */
	  prevradio=list;
	prevradio->selected=DIALOG_B_ENABLED;
    }
		

    /* check_x is the position of the check-tag pair, item_x of the text */
    {
	int check_offset = (desc->type==DIALOG_MENU ? 2 : 6);
	i = check_x + check_offset; /*offset*/
	check_x = (list_width - check_offset - check_x -item_x) / 2;
	item_x = check_x + i;
    }
    
    /* Print the list, and set first/last */
    first=curr=list;
    for (i=0, new=list; new && i<max_choice; i++, last=new, new=new->next)
	print_item (subw, desc->type, new);
    first=list;

    /* register the new window, along with its borders */
    _dialog_mkbigregion (box_y, box_x, list_height + 2, list_width + 2,
		     item_no, item_x /* the threshold */ , 0 /* normal */ );

    if (list_height < item_no)
      DIALOG_DOWNARROW(dialog, box_y + list_height + 1, box_x + check_x + 3);

    /* Ok, now restore the cursor... */
    wmove(dialog, curs_y, curs_x);
    wnoutrefresh (subw);
    wrefresh(dialog);

    x = desc->wid / 2 - 11; /* what? */
    y = desc->hei - 2;


    while ( (key=_dialog_wgetch (dialog)) != DIALOG_ESC) {

	/* fprintf(stderr,"key 0x%04x (%03d)\n",key,key); */

	if (key>='A' && key <='Z')
	  key=tolower(key);

	/*
	 * Look for an hotkey, either in the visible list or everywhere
	 * Giving precedence to the list over the button is safer
	 */

	for (i=0, new=first; new && i<max_choice; i++, new=new->next)
	  if (tolower(*new->hotkey)==key)
	    break;
	if (i==max_choice) new=NULL;

	if (!new)
	  for (new=list; new; new=new->next)
	    if (tolower(*new->hotkey)==key)
	      break;

	/*
	 * Look for a button hotkey
	 */
	if (!new)
	  for (i=0, bptr=desc->buttons; i<desc->buttnum; i++, bptr++) {
	      if ( key==(int)tolower(*bptr->hotkey) ||
		  key==DIALOG_M_EVENT+(int)toupper(*bptr->hotkey) ) {
		  delwin(dialog);
		  return(i);
	      }
	      if (key == DIALOG_M_EVENT+(int)tolower(*bptr->hotkey)) {
		  new = curr; button = i;
		  break;
	      }
	  }

	/*
	 * Decode digits and moused digits
	 */

	if (!new) {
	    if ( (key>='1' && key<='9')
		|| (key>=DIALOG_M_EVENT && key-DIALOG_M_EVENT<' ') ) {
		key -= (key>DIALOG_M_EVENT ? DIALOG_M_EVENT : '1');
		key=min(key,max_choice);
		for (new=first; key && new->next; new=new->next)
		  key--;
	    }
	}

	if (!new) { /* still missing, then switch */
	    new=curr;
	    switch(key) {
	      case KEY_UP:
	      case '-':
		if (curr->prev) new=curr->prev; break;

	      case KEY_DOWN: 
	      case '+':
		if (curr->next) new=curr->next; break;

	      case KEY_NPAGE:
		new=curr;
		for (i=0; i<max_choice && new->next; new=new->next, i++)
		  ;
		break;

	      case KEY_PPAGE:
		new=curr;
		for (i=0; i<max_choice && new->prev; new=new->prev, i++)
		  ;
		break;


	      case ' ':
		switch(desc->type) { /* here the three types are different */
		  case DIALOG_CHECK:
		    if (curr->selected)
		      curr->selected=DIALOG_B_DISABLED;
		    else
		      curr->selected=DIALOG_B_ENABLED;
		    break;

		  case DIALOG_RADIO:
		    if (prevradio!=curr) {
			prevradio->selected=DIALOG_B_DISABLED;
			if (prevradio->index >= first->index
			    && prevradio->index <= last->index)
			  print_item(subw,desc->type,prevradio);
			prevradio=curr;
		    }
		    curr->selected=DIALOG_B_ENABLED;
		    break;

		  case DIALOG_MENU:
		    curr->selected=DIALOG_B_ENABLED;
		    delwin(dialog);
		    return button;
		}
		print_item(subw,desc->type,curr);
		break;

	      case DIALOG_TAB:
	      case KEY_LEFT:
		button--; if (button<0) button=desc->buttnum-1; break;

	      case KEY_RIGHT:
		button++; button %=  desc->buttnum; break;

	      case '\n':
		if (desc->type == DIALOG_MENU)
		  curr->selected=DIALOG_B_ENABLED;
		delwin (dialog);
		return button;

	    }
	}

	/*
	 * Ok, we're done. Now just go redraw it. First the list, either
	 * curr and new (no scroll) or all of them. Then the arrows, and
	 * finally the buttons. Thus the cursor remains on the last button.
	 */
	if (new && new!=curr) {
	    Dialog_list *old=curr; curr=new;

	    if (curr->index >= first->index && curr->index <= last->index) {
		print_item(subw,desc->type,old);
		print_item(subw,desc->type,curr);
	    } else {
		int index=curr->index-(max_choice/2); /* the first one */
		if (index+list_height>item_no)
		  index=item_no-list_height;
		if (index<0)
		  index=0;
		for (first=list; first->index<index; first=first->next)
		  ;
		for (i=0, new=first; new && i<max_choice;
		     i++, last=new, new=new->next)
		  print_item (subw, desc->type, new);
	    }

	    if (first->prev)
	      DIALOG_UPARROW(dialog, box_y, box_x+check_x+3);
	    else
	      DIALOG_NOARROW(dialog, box_y, box_x+check_x+3);
	    if (last->next)
	      DIALOG_DOWNARROW(dialog, box_y+list_height+1, box_x+check_x+3);
	    else
	      DIALOG_NOARROW(dialog, box_y+list_height+1, box_x+check_x+3);
	    
	}


	if (desc->buttons[button].status==DIALOG_B_DISABLED) {
	    desc->buttons[button].status=DIALOG_B_ENABLED;
	    _dialog_print_buttons(desc);
	    getyx(dialog, curs_y, curs_x);
	}
	wmove(dialog, curs_y, curs_x); /* cursor on button */
	wnoutrefresh(subw);
	wrefresh(dialog);
    }
    delwin (dialog);
    return -1;			/* ESC pressed */
}



/**** Local Variables:  ****/
/**** tab-width:8       ****/
/**** c-indent-level:4  ****/
/**** c-label-offset:-2 ****/
/**** End:              ****/

